% Hua-sheng XIE, huashengxie@gmail.com, IFTS-ZJU, 2013-04-18 19:09
% pdrk_es1d.m, Plasma Dispersion Relastion solver, Kinetic, ES1D
% transform to matrix eigenvalue problem X=MX
% Landau damping: 2*k^2-Z'()=0, =w/(k*sqrt(2))
% eight-pole approximation for Z'()
%     =sum {aj/(bj-)^2+conj(aj)/(conj(bj)+)^2}
% test OK

% 2014-05-24 22:40, rewrite, 
close all; clear; clc;

%% read input parameters
% par = importdata('pdrk_landau.dat', ' ', 1); 
% par = importdata('pdrk_acoustic.dat', ' ', 1); 
par = importdata('pdrk_beam.dat', ' ', 1); 
[S, col]=size(par.data);
if(col~=5)
    disp('Wrong input data !!!');
end

for j=1:S
    qs(j)=par.data(j,1);
    ms(j)=par.data(j,2);
    ns0(j)=par.data(j,3);
    Ts(j)=par.data(j,4);
    vs0(j)=par.data(j,5);
end
vts=sqrt(2*Ts./ms);
lambdaDs=sqrt(Ts./(ns0.*qs.^2));
kDs=1./lambdaDs;
wps=sqrt(ns0.*qs.^2./ms); % plasma frequency


%%
% Landau damping data from Z function
data = [0.1   1.0152   -4.75613E-15
0.2   1.06398   -5.51074E-05
0.3   1.15985   -0.0126204
0.4   1.28506   -0.066128
0.5   1.41566   -0.153359
0.6   1.54571   -0.26411
0.7   1.67387   -0.392401
0.8   1.7999   -0.534552
0.9   1.92387   -0.688109
1.0   2.0459   -0.85133
1.5   2.63233   -1.77571
2   3.18914   -2.8272];
id=2;
% id=10;
k=data(id,1) % give k
wr=data(id,2)
wi=data(id,3)

%%
% clear; 
J=4;
if(J==8)
    % Ronnmark1982, 8-pole for Z function, and Z'
    bzj(1)=-1.734012457471826E-2-4.630639291680322E-2i;
    bzj(2)=-7.399169923225014E-1+8.395179978099844E-1i;
    bzj(3)=5.840628642184073+9.536009057643667E-1i;
    bzj(4)=-5.583371525286853-1.120854319126599E1i;
    czj(1)=2.237687789201900-1.625940856173727i;
    czj(2)=1.465234126106004-1.789620129162444i;
    czj(3)=.8392539817232638-1.891995045765206i;
    czj(4)=.2739362226285564-1.941786875844713i;

    bzj(5:8)=conj(bzj(1:4));
    czj(5:8)=-conj(czj(1:4));
elseif(J==4)
    % Martin1980, 4-pole
%     bzj(1)=0.5468-0.0372i;
%     bzj(2)=-1.0468+2.1018i;
%     czj(1)=-1.2359-1.2150i;
%     czj(2)=-0.3786-1.3509i;
    bzj(1)=0.546796859834032 + 0.037196505239277i;
    bzj(2)=-1.046796859834027 + 2.101852568038518i;
    czj(1)=1.23588765343592 - 1.21498213255731i;
    czj(2)=-0.378611612386277 - 1.350943585432730i;
    
    bzj(3:4)=conj(bzj(1:2));
    czj(3:4)=-conj(czj(1:2));
elseif(J==3)
    % Martin1980, 3-pole
    bzj(1)=0.1822+0.5756i;
    bzj(2)=-1.3643;
    czj(1)=-0.9217-0.9091i;
    czj(2)=-1.0204i;
    
    bzj(3)=conj(bzj(1));
    czj(3)=-conj(czj(1));
    
elseif(J==2)
    % Huba2009, 2-pole
    bzj(1)=-(0.5+0.81i);
    czj(1)=0.51-0.81i;
    bzj(2)=conj(bzj(1));
    czj(2)=-conj(czj(1));   
    
else
    % Martin1979, 2-pole
    bzj(1)=-(0.5+1.2891i);
    czj(1)=0.5138-1.0324i;
    bzj(2)=conj(bzj(1));
    czj(2)=-conj(czj(1));
end
J=length(bzj)
sum(bzj)
sum(bzj.*czj)
%%

SJ=S*J;

sj=0;
aSJ=1;
M=zeros(SJ,SJ);
for s=1:S
%     aSJ=aSJ+(kDs(s)^2/k^2).*sum(bzj);
    for j=1:J;
        sj=sj+1;
        csj(sj)=k*(czj(j)*vts(s)+vs0(s));
        bsj(sj)=vts(s)*bzj(j)*czj(j)*(kDs(s)^2/k);
%         bsj(sj)=-(kDs(s)^2/k^2)*bzj(j)*csj(sj);
%         aSJ=aSJ-(kDs(s)^2/k^2)*(bzj(j));
    end
end

for sj=1:SJ
    M(sj,:)=-bsj(sj);
%     M(sj,sj)=bsj(sj)+aSJ*csj(sj);
    M(sj,sj)=-bsj(sj)+csj(sj);
end


d=eig(M);
omega=d;
% ind=find(imag(omega)==max(imag(omega)));
% w=omega(ind)
[wi,ind]=sort(imag(omega),'descend');
w=omega(ind)

if(J==8)
    wJ8=omega;
    save('wdat', 'wJ8');
end

%% plot Landau solutions
% h=figure('unit','normalized','Position',[0.01 0.57 0.3 0.35]);
figure;
set(gcf,'DefaultAxesFontSize',15);
load('wdat');


% plot contour of the Landau solution
wrmax=4; wimax=5.0;
vc = 0:.02:0.2;
[Rew,Imw] = meshgrid(-wrmax:.01:wrmax, -wimax:.01:0);
f1 = 1 + k^2 + 1i*(1/k)*sqrt(pi/2).*(Rew +1i*Imw).*faddeeva((Rew+1i*Imw)/(sqrt(2)*k));
f = sqrt(real(f1).^2 + imag(f1).^2);
contour(Rew, Imw, f, vc);  hold on;


plot(real(omega),imag(omega),'rx','LineWidth',2,'MarkerSize',10); hold on;
% plot(real(omega),imag(omega),'gx',real(wJ8),imag(wJ8),'r+',...
%     'LineWidth',2,'MarkerSize',10); hold on;

legend('Matrix solutions','Z(\zeta) solutions');
% legend('Z(\zeta) solutions','Matrix J=4','Matrix J=8');
legend('boxoff');
ylim([-wimax,wimax/2]); xlim([-wrmax,wrmax]);

% title(['k=',num2str(k)]);
text(-3,1.5,['k=',num2str(k)],'FontSize',18);
xlabel('\omega_r'); ylabel('\omega_i'); 

print('-dpng',['pdrk_es1d_k',num2str(k),'_J',num2str(J),'.png']);


